
/***************************************************************************
 *   This file is part of Aspect, a simple PEC tool.                       *
 *                                                                         *
 *   Copyright (C) 2007 by Wolfgang Hoffmann <woho@woho.de>                *
 *                                                                         *
 *   This program is free software, licensed under the GPL v2.             *
 *   See the file COPYING for more details.                                *
 ***************************************************************************/


#include "settings.h"
#include "ui_settings.h"
#include "log.h"

#include <QtGui>




/********************************************************************/
/*!
    \class Settings
    \brief Container for application settings

*/

Settings Settings::g_oSettings;


Settings::Settings()
:   m_bFrozen(false),
    m_bChanged(false)
    {
    QCoreApplication::setOrganizationName("woho");
    QCoreApplication::setOrganizationDomain("woho.de");
    QCoreApplication::setApplicationName("aspect");
    m_pQSettings = new QSettings(QSettings::IniFormat, QSettings::UserScope,
        QString::fromUtf8("woho"), QString::fromUtf8("aspect"));

    // remove legacy setting from pre-v4.00rc5 times
    m_pQSettings->remove("pecGen/accel");
    }

Settings::~Settings()
    {
    delete m_pQSettings;
    }

void Settings::emitChanged()
    {
    g_oSettings.emitChangedInternal(true);
    }

void Settings::connect(QObject *pReceiver, const char *szSlot)
    {
    QObject::connect(&g_oSettings, SIGNAL(settingsChanged()), pReceiver, szSlot);
    }

void Settings::freeze()
    {
    g_oSettings.m_bFrozen = true;
    }

void Settings::thaw()
    {
    g_oSettings.m_bFrozen = false;
    g_oSettings.emitChangedInternal(false);
    }

void Settings::emitChangedInternal(bool bFromChange)
    {
    if (bFromChange)
        m_bChanged = true;
    if (m_bChanged && (!m_bFrozen))
        {
        m_bChanged = false;
        emit settingsChanged();
        }
    }


QString Settings::getPathLog()
    {
    return QDir::toNativeSeparators(settings()
        .value("path/log", QDir::tempPath()).toString());
    }

void Settings::setPathLog(const QString &qsPath)
    {
    if (qsPath == getPathLog())
        return;
    QString qsPathSep = QDir::fromNativeSeparators(qsPath);
    Settings::settings().setValue("path/log", qsPathSep);
    emitChanged();
    }


QStringList Settings::getPathHistory()
    {
    QStringList qslHistorySep = settings()
        .value("path/history", QStringList()).toStringList();
    QStringList qslHistory;
    for (int nCnt = 0; nCnt < qslHistorySep.size(); nCnt++)
        qslHistory.append(QDir::toNativeSeparators(qslHistorySep.at(nCnt)));
    return qslHistory;
    }

QString Settings::getPathDefault()
    {
    return QDir::toNativeSeparators(settings()
        .value("path/current", QDir::home().absolutePath()).toString());
    }

void Settings::setPathDefault(const QString &qsPath)
    {
    if (qsPath == getPathDefault())
        return;
    QString qsPathSep = QDir::fromNativeSeparators(qsPath);
    Settings::settings().setValue("path/current", qsPathSep);
    // add current path to begin of history
    // remove any duplicates and keep list below 20 entries
    // don't use Settings::getPathHistory() here, but query
    // directly to get list with non-native separators.
    QStringList qslHistorySep = settings()
        .value("path/history", QStringList()).toStringList();
    for (int nCnt = qslHistorySep.size() - 1; nCnt >= 0; nCnt--)
        if (qslHistorySep.at(nCnt) == qsPathSep)
            qslHistorySep.removeAt(nCnt);
    qslHistorySep.prepend(qsPathSep);
    while (qslHistorySep.size() > 20)
        qslHistorySep.removeLast();
    Settings::settings().setValue("path/history", qslHistorySep);
    emitChanged();
    }


bool Settings::getLoggingGeneral()
    {
    return settings().value("log/general", true).toBool();
    }

void Settings::setLoggingGeneral(bool bOn)
    {
    if (bOn == getLoggingGeneral())
        return;
    Settings::settings().setValue("log/general", bOn);
    emitChanged();
    }

bool Settings::getLoggingMcu()
    {
    return settings().value("log/mcu", false).toBool();
    }

void Settings::setLoggingMcu(bool bOn)
    {
    if (bOn == getLoggingMcu())
        return;
    Settings::settings().setValue("log/mcu", bOn);
    emitChanged();
    }


QString Settings::getMcuPort()
    {
    return settings().value("mcu/serialport", QString::fromUtf8("COM1")).toString();
    }

void Settings::setMcuPort(const QString &qsPort)
    {
    if (qsPort == getMcuPort())
        return;
    Settings::settings().setValue("mcu/serialport", qsPort);
    emitChanged();
    }

int Settings::getMcuFirmware()
    {
    return settings().value("mcu/firmware", 35904).toInt() / 100;
    }

void Settings::setMcuFirmware(int nFirmware)
    {
    // store as ((major*100)+minor)*100+revision
    // revision is currently not considered
    int nCurrent = getMcuFirmware();
    if (nFirmware < 100)
        {
        nFirmware *= 100;
        nCurrent = (nCurrent / 100) * 100;
        }
    if (nFirmware == nCurrent)
        return;
    Settings::settings().setValue("mcu/firmware", nFirmware * 100);
    emitChanged();
    }

int Settings::getMcuWormWheelTeeth()
    {
    return settings().value("mcu/wormwheelteeth", 180).toInt();
    }

void Settings::setMcuWormWheelTeeth(int nTeeth)
    {
    if (nTeeth == getMcuWormWheelTeeth())
        return;
    Settings::settings().setValue("mcu/wormwheelteeth", nTeeth);
    emitChanged();
    }


int Settings::getPecGenTraversal()
    {
    return settings().value("pecGen/traversal", -1).toInt();
    }

void Settings::setPecGenTraversal(int nTraversal)
    {
    if (nTraversal == getPecGenTraversal())
        return;
    if (nTraversal == -1)
        Settings::settings().remove("pecGen/traversal");
    else
        Settings::settings().setValue("pecGen/traversal", nTraversal);
    emitChanged();
    }

double Settings::getPecGenAccel()
    {
    return settings().value("pecGen/acceleration", 200.).toDouble();
    }

void Settings::setPecGenAccel(double dfAccel)
    {
    if (dfAccel == getPecGenAccel())
        return;
    if (dfAccel == 200.)
        Settings::settings().remove("pecGen/acceleration");
    else
        Settings::settings().setValue("pecGen/acceleration", dfAccel);
    emitChanged();
    }

int Settings::getPecGenAggressMin()
    {
    return settings().value("pecGen/aggressMin", 3).toInt();
    }

void Settings::setPecGenAggressMin(int nAggressMin)
    {
    if (nAggressMin == getPecGenAggressMin())
        return;
    if (nAggressMin == 3)
        Settings::settings().remove("pecGen/aggressMin");
    else
        Settings::settings().setValue("pecGen/aggressMin", nAggressMin);
    emitChanged();
    }

int Settings::getPecGenAggressMax()
    {
    return settings().value("pecGen/aggressMax", 15).toInt();
    }

void Settings::setPecGenAggressMax(int nAggressMax)
    {
    if (nAggressMax == getPecGenAggressMax())
        return;
    if (nAggressMax == 15)
        Settings::settings().remove("pecGen/aggressMax");
    else
        Settings::settings().setValue("pecGen/aggressMax", nAggressMax);
    emitChanged();
    }

int Settings::getPecGenFactorMax()
    {
    return settings().value("pecGen/factorMax", 15).toInt();
    }

void Settings::setPecGenFactorMax(int nFactorMax)
    {
    if (nFactorMax == getPecGenFactorMax())
        return;
    if (nFactorMax == 15)
        Settings::settings().remove("pecGen/factorMax");
    else
        Settings::settings().setValue("pecGen/factorMax", nFactorMax);
    emitChanged();
    }


bool Settings::getExperimentalTabProfile()
    {
    return settings().value("guiExperimental/tabProfile", false).toBool();
    }

void Settings::setExperimentalTabProfile(bool bEnable)
    {
    if (bEnable == getExperimentalTabProfile())
        return;
    if (!bEnable)
        Settings::settings().remove("guiExperimental/tabProfile");
    else
        Settings::settings().setValue("guiExperimental/tabProfile", true);
    emitChanged();
    }

bool Settings::getExperimentalMenuMount()
    {
    return settings().value("guiExperimental/menuMount", false).toBool();
    }

void Settings::setExperimentalMenuMount(bool bEnable)
    {
    if (bEnable == getExperimentalMenuMount())
        return;
    if (!bEnable)
        Settings::settings().remove("guiExperimental/menuMount");
    else
        Settings::settings().setValue("guiExperimental/menuMount", true);
    emitChanged();
    }




/********************************************************************/
/*!
    \class SettingsDialog
    \brief GUI for application settings

    \todo implement logfile path selection
    \todo implement settings export/import
*/

SettingsDialog::SettingsDialog(QWidget *pParent)
:   QDialog(pParent)
    {
    setAttribute(Qt::WA_DeleteOnClose, false);
    setAttribute(Qt::WA_QuitOnClose, false);

    m_pUi = new Ui::SettingsDialog;
    m_pUi->setupUi(this);

    m_pUi->logfilesPath->setText(
        Settings::settings().value("path/logfiles", QDir::tempPath()).toString());

    m_pUi->pecGenTraversal->setCurrentIndex((Settings::getPecGenTraversal() > 0)? 1: 0);
    m_pUi->pecGenAccel->setValue(Settings::getPecGenAccel());
    m_pUi->pecGenAggressMin->setValue(Settings::getPecGenAggressMin());
    m_pUi->pecGenAggressMax->setValue(Settings::getPecGenAggressMax());
    m_pUi->pecGenFactor->setValue(Settings::getPecGenFactorMax());

    m_pUi->expTabProfile->setChecked(Settings::getExperimentalTabProfile());
    m_pUi->expMenuMount->setChecked(Settings::getExperimentalMenuMount());

    // todo: implement functionality
    m_pUi->settingsImport->hide();
    m_pUi->settingsExport->hide();
    }


SettingsDialog::~SettingsDialog()
    {
    delete m_pUi;
    }


void SettingsDialog::on_logfilesGeneral_clicked()
    {
    Settings::setLoggingGeneral(m_pUi->logfilesGeneral->isChecked());
    }


void SettingsDialog::on_logfilesMcu_clicked()
    {
    Settings::setLoggingMcu(m_pUi->logfilesMcu->isChecked());
    }


void SettingsDialog::on_logfilesCopy_clicked()
    {
    LOG(4, "SettingsDialog::on_logfilesCopy_clicked");

    QStringList qslFileNames;
    {
    QFileDialog oSrcDialog(this);
    oSrcDialog.setAcceptMode(QFileDialog::AcceptOpen);
    oSrcDialog.setFileMode(QFileDialog::ExistingFiles);
    oSrcDialog.setViewMode(QFileDialog::Detail);
    oSrcDialog.setWindowTitle(tr("Aspect - Select Logfiles to Copy"));
    oSrcDialog.setDirectory(Settings::getPathLog());
    QStringList qslFilters;
    qslFilters << tr("Log File (aspect_*.txt)");
    qslFilters << tr("All Files (*)");
    oSrcDialog.setFilters(qslFilters);
    oSrcDialog.setDefaultSuffix(QString::fromUtf8("csv"));
    if (!oSrcDialog.exec())
        return;
    qslFileNames = oSrcDialog.selectedFiles();
    if (qslFileNames.size() <= 0)
        return;
    }

    QDir oDstDir;
    {
    QFileDialog oDstDialog(this);
    oDstDialog.setAcceptMode(QFileDialog::AcceptSave);
    oDstDialog.setFileMode(QFileDialog::DirectoryOnly);
    oDstDialog.setViewMode(QFileDialog::Detail);
    oDstDialog.setWindowTitle(tr("Aspect - Select Destination Path"));
    oDstDialog.setHistory(Settings::getPathHistory());
    oDstDialog.setDirectory(Settings::getPathDefault());
    QStringList qslFilters;
    qslFilters << tr("Directories (*)");
    oDstDialog.setFilters(qslFilters);
    if (!oDstDialog.exec())
        return;
    oDstDir = oDstDialog.directory();
    }

    {
    QProgressDialog oProgress(tr("Copying logfiles ..."),
        tr("Cancel"), 0, 1000, this);
    oProgress.setWindowTitle(tr("Aspect - Copy logfiles"));
    oProgress.setMinimumDuration(0);
    oProgress.setWindowModality(Qt::WindowModal);
    QApplication::setOverrideCursor(Qt::BusyCursor);
    LOG(1, "Copy logfiles to %s", LQS(oDstDir.absoluteFilePath(QString())));
    for (int nFile = 0; nFile < qslFileNames.size(); nFile++)
        {
        QString qsSrc = qslFileNames.at(nFile);
        LOG(1, "    %s", LQS(qsSrc));
        QString qsDst = oDstDir.absoluteFilePath(QFileInfo(qsSrc).fileName());
        oProgress.setLabelText(tr("Copying logfiles ...\n\n%1\n%2")
            .arg(qsSrc).arg(qsDst));
        oProgress.setValue(nFile * 1000 / qslFileNames.size());
        if (QFile::exists(qsDst))
            QFile::remove(qsDst);
        if (!QFile::copy(qsSrc, qsDst))
            LOG(1, "        failed.");
        qApp->processEvents();
        if (oProgress.wasCanceled())
            break;
        }
    QApplication::restoreOverrideCursor();
    }
    }


void SettingsDialog::on_pecGenTraversal_activated(int nIndex)
    {
    Settings::setPecGenTraversal((nIndex == 0)? -1: 1);
    }


void SettingsDialog::on_pecGenAccel_valueChanged(double dfValue)
    {
    Settings::setPecGenAccel(dfValue);
    }


void SettingsDialog::on_pecGenAggressMin_valueChanged(int nValue)
    {
    Settings::setPecGenAggressMin(nValue);
    }


void SettingsDialog::on_pecGenAggressMax_valueChanged(int nValue)
    {
    Settings::setPecGenAggressMax(nValue);
    }


void SettingsDialog::on_pecGenFactor_valueChanged(int nValue)
    {
    Settings::setPecGenFactorMax(nValue);
    }


void SettingsDialog::on_expTabProfile_clicked(bool bChecked)
    {
    Settings::setExperimentalTabProfile(bChecked);
    }


void SettingsDialog::on_expMenuMount_clicked(bool bChecked)
    {
    Settings::setExperimentalMenuMount(bChecked);
    }

